@@ -6,7 +6,7 @@ use strict;
 use 5.010001;
 no warnings 'utf8';
 
-our $VERSION = '0.035';
+our $VERSION = '0.036';
 
 use Encode                qw( decode );
 use File::Basename        qw( basename );
@@ -55,9 +55,8 @@ sub new {
         _continue => '  CONTINUE',
         _info     => '  INFO',
         _reset    => '  RESET',
-        yes_no            => [ 'NO', 'YES' ],
         line_fold         => { Charset=> 'utf8', OutputCharset => '_UNICODE_', Urgent => 'FORCE' },
-        sect_generic      => 'AAA_generic',
+        sect_generic      => 'Generic',
         stmt_init_tab     => 4,
         tbl_info_width    => 140,
         avail_aggregate   => [ "AVG(X)", "COUNT(X)", "COUNT(*)", "MAX(X)", "MIN(X)", "SUM(X)" ],
@@ -65,71 +64,14 @@ sub new {
         avail_operators   => [ "REGEXP", "NOT REGEXP", "LIKE", "NOT LIKE", "IS NULL", "IS NOT NULL", "IN", "NOT IN",
                             "BETWEEN", "NOT BETWEEN", " = ", " != ", " <> ", " < ", " > ", " >= ", " <= ", "LIKE col",
                             "NOT LIKE col", "LIKE %col%", "NOT LIKE %col%", " = col", " != col", " <> col", " < col",
-                            " > col", " >= col", " <= col" ],
-                            # "LIKE col%", "NOT LIKE col%", "LIKE %col", "NOT LIKE %col",
+                            " > col", " >= col", " <= col" ], # "LIKE col%", "NOT LIKE col%", "LIKE %col", "NOT LIKE %col"
         avail_db_drivers  => [ 'SQLite', 'mysql', 'Pg' ],
         hidd_func_pr      => { Epoch_to_Date => 'DATE', Truncate => 'TRUNC', Epoch_to_DateTime => 'DATETIME',
                                Bit_Length => 'BIT_LENGTH', Char_Length => 'CHAR_LENGTH' },
         keys_hidd_func_pr => [ qw( Epoch_to_Date Bit_Length Truncate Char_Length Epoch_to_DateTime ) ],
+        connect_opt_pre   => {  SQLite => 'sqlite_', mysql => 'mysql_', Pg => 'pg_' },
     };
 
-    $info->{defaults} = {
-        db_defaults          => 'Dummy',
-        db_drivers           => [ 'SQLite', 'mysql', 'Pg' ],
-        db_login_once        => 0,
-        _env_dbi             => 1,
-        env_dbi_user         => 0,
-        env_dbi_pass         => 0,
-        _enchant             => 'Dummy',
-        keep_db_choice       => 0,
-        keep_schema_choice   => 0,
-        keep_table_choice    => 0,
-        table_expand         => 1,
-        keep_header          => 0,
-        lock_stmt            => 0,
-        max_rows             => 50_000,
-        metadata             => 0,
-        mouse                => 0,
-        min_col_width        => 30,
-        operators            => [ "REGEXP", " = ", " != ", " < ", " > ", "IS NULL", "IS NOT NULL" ],
-        _parentheses         => 'Dummy',
-        w_parentheses        => 0,
-        h_parentheses        => 0,
-        progress_bar         => 20_000,
-        regexp_case          => 0,
-        sssc_mode            => 0,
-        tab_width            => 2,
-        undef                => '',
-        binary_string        => 'BNRY',
-        thsd_sep             => ',',
-        header_row           => 1,
-        choose_columns       => 0,
-        db_browser_mode      => 1,
-        SQLite => {
-            _reset_cache_cmdline_only  => 0,
-            sqlite_unicode             => 1,
-            sqlite_see_if_its_a_number => 1,
-            _binary_filter             => 0,
-            _sqlite_search_dir         => undef,
-        },
-        mysql => {
-            host                => '',
-            port                => '',
-            mysql_enable_utf8   => 1,
-            _binary_filter      => 0,
-        },
-        Pg => {
-            host                => '',
-            port                => '',
-            pg_enable_utf8      => 1,
-            _binary_filter      => 0,
-        },
-    };
-
-    $info->{SQLite}{options} = [ qw( sqlite_unicode sqlite_see_if_its_a_number _binary_filter ) ];
-    $info->{mysql}{options}  = [ qw( mysql_enable_utf8 _binary_filter host port ) ];
-    $info->{Pg}{options}     = [ qw( pg_enable_utf8 _binary_filter host port ) ];
-
     return bless { info => $info }, $class;
 }
 
@@ -146,31 +88,29 @@ sub __init {
     my $app_dir = $my_data ? catdir( $my_data, 'db_browser_conf' ) : catdir( $home, '.db_browser_conf' );
     mkdir $app_dir or die $! if ! -d $app_dir;
     $self->{info}{app_dir}       = $app_dir;
-    $self->{info}{config_file}   = catfile $app_dir, 'config.json';
+    $self->{info}{conf_file_fmt} = catfile $app_dir, 'config_%s.json';
     $self->{info}{db_cache_file} = catfile $app_dir, 'cache_db_search.json';
-    $self->{info}{defaults}{SQLite}{_sqlite_search_dir} = [ $home ];
 
-    my $opt = $self->{info}{defaults};
     if ( ! eval {
-        my $obj_opt = App::DBBrowser::Opt->new( $self->{info}, $opt );
-        $opt = $obj_opt->read_config_file( $self->{info}{config_file} );
+        my $obj_opt = App::DBBrowser::Opt->new( $self->{info}, $self->{opt} );
+        $self->{opt} = $obj_opt->read_config_files();
         my $help;
         GetOptions (
             'h|?|help' => \$help,
-            's|search' => \$opt->{SQLite}{_reset_cache_cmdline_only},
+            's|search' => \$self->{info}{sqlite_search},
         );
         if ( $help ) {
-            if ( $opt->{mouse} ) {
+            if ( $self->{opt}{mouse} ) {
                 for my $key ( keys %{$self->{info}} ) {
                     next if $key !~ /^lyt_/;
-                    $self->{info}{$key}{mouse} = $opt->{mouse};
+                    $self->{info}{$key}{mouse} = $self->{opt}{mouse};
                 }
             }
-            $opt = $obj_opt->options( $opt );
-            if ( defined $opt->{mouse} ) {
+            $self->{opt} = $obj_opt->set_options();
+            if ( defined $self->{opt}{mouse} ) {
                 for my $key ( keys %{$self->{info}} ) {
                     next if $key !~ /^lyt_/;
-                    $self->{info}{$key}{mouse} = $opt->{mouse};
+                    $self->{info}{$key}{mouse} = $self->{opt}{mouse};
                 }
             }
         }
@@ -178,9 +118,10 @@ sub __init {
     ) {
         say 'Configfile/Options:';
         $self->__print_error_message( $@ );
-        $opt = $self->{info}{defaults};
+        my $obj_opt = App::DBBrowser::Opt->new( $self->{info}, $self->{opt} );
+        $self->{opt} = $obj_opt->defaults();
+        shift @ARGV while $ARGV[0] =~ /^-/;
     }
-    $self->{opt} = $opt;
     if ( $self->{opt}{mouse} ) {
         for my $key ( keys %{$self->{info}} ) {
             next if $key !~ /^lyt_/;
@@ -188,10 +129,8 @@ sub __init {
         }
     }
     $self->{info}{ok} = '<OK>' if $self->{opt}{sssc_mode};
-    $self->{info}{sqlite_search} = 0;
-    $self->{info}{sqlite_search} = 1 if @ARGV || $self->{opt}{SQLite}{_reset_cache_cmdline_only};
-    $self->{info}{sqlite_search} = 0 if @{$self->{opt}{db_drivers}} == 1 && $self->{opt}{db_drivers}[0] eq 'SQLite';
-    $self->{info}{sqlite_dirs} = @ARGV ? \@ARGV : $self->{opt}{SQLite}{_sqlite_search_dir};
+    $self->{info}{argv} = @ARGV ? 1 : 0;
+    $self->{info}{sqlite_dirs} = @ARGV ? \@ARGV : $self->{opt}{SQLite}{dirs_sqlite_search} // [ $home ];
 }
 
 
@@ -203,9 +142,9 @@ sub run {
 
     DB_DRIVER: while ( 1 ) {
 
-        if ( $self->{info}{sqlite_search} ) {
+        if ( $self->{info}{sqlite_search} || $self->{info}{argv} ) {
             $db_driver = 'SQLite';
-            $self->{info}{sqlite_search} = 0;
+            $self->{info}{argv} = 0;
         }
         else {
             if ( @{$self->{opt}{db_drivers}} == 1 ) {
@@ -245,6 +184,7 @@ sub run {
         }
         if ( ! @$databases ) {
             $self->__print_error_message( "no $db_driver-databases found\n" );
+            exit if @{$self->{opt}{db_drivers}} == 1;
             next DB_DRIVER;
         }
         my $regexp_system;
@@ -299,7 +239,7 @@ sub run {
                     last DB_DRIVER if   $self->{info}{one_db_driver};
                     next DB_DRIVER if ! $self->{info}{one_db_driver};
                 }
-                if ( $self->{opt}{keep_db_choice} ) {
+                if ( $self->{opt}{menus_memory} ) {
                     if ( $old_idx_db == $idx_db ) {
                         $old_idx_db = 0;
                         next DATABASE;
@@ -319,7 +259,7 @@ sub run {
                 1 }
             ) {
                 say 'Get database handle and schema names:';
-                delete $self->{info}{login}{$db_driver}{$db};
+                delete $self->{info}{login}{$db_driver . '_' . $db};
                 $self->__print_error_message( $@ );
                 # remove database from @databases
                 next DATABASE;
@@ -361,7 +301,7 @@ sub run {
                     );
                     $schema = $choices->[$idx_sch] if defined $idx_sch;
                     next DATABASE if ! defined $schema;
-                    if ( $self->{opt}{keep_schema_choice} ) {
+                    if ( $self->{opt}{menus_memory} ) {
                         if ( $old_idx_sch == $idx_sch ) {
                             $old_idx_sch = 0;
                             next SCHEMA;
@@ -425,7 +365,7 @@ sub run {
                         next SCHEMA if defined $data->{$db}{schemas} && @{$data->{$db}{schemas}} > 1;
                         next DATABASE;
                     }
-                    if ( $self->{opt}{keep_table_choice} ) {
+                    if ( $self->{opt}{menus_memory} ) {
                         if ( $old_idx_tbl == $idx_tbl ) {
                             $old_idx_tbl = 0;
                             next TABLE;
@@ -505,8 +445,10 @@ sub run {
                                 push @$pr_columns, $col;
                             }
                         }
-                        $self->{opt}{binary_filter} =    $self->{opt}{$db_driver . '_' . $db}{_binary_filter}
-                                                      || $self->{opt}{$db_driver}{_binary_filter};
+
+                        $self->{opt}{_db_browser_mode} = 1;
+                        $self->{opt}{binary_filter}    =    $self->{opt}{$db_driver . '_' . $db}{binary_filter}
+                                                         || $self->{opt}{$db_driver}{binary_filter};
 
                         PRINT_TABLE: while ( 1 ) {
                             my $all_arrayref = $self->__read_table( $sql, $dbh, $table, $select_from_stmt, $qt_columns, $pr_columns );
@@ -546,8 +488,9 @@ sub __available_databases_cached {
     my $cache_key = $db_driver . '_' . join ' ', @{$self->{info}{sqlite_dirs}};
     my $obj_opt = App::DBBrowser::Opt->new( $self->{info}, $self->{opt} );
     $self->{info}{cache} = $obj_opt->read_json( $self->{info}{db_cache_file} );
-    if ( $self->{opt}{$db_driver}{_reset_cache_cmdline_only} ) {
+    if ( $self->{info}{sqlite_search} ) {
         delete $self->{info}{cache}{$cache_key};
+        $self->{info}{sqlite_search} = 0;
     }
     if ( $self->{info}{cache}{$cache_key} ) {
         $self->{info}{cached} = ' (cached)';
@@ -1279,10 +1222,10 @@ sub __read_table {
                 my @pre = ( $self->{info}{ok} );
                 unshift @pre, undef if $self->{opt}{sssc_mode};
                 my @choices = @cols;
-                if ( $self->{opt}{w_parentheses} == 1 ) {
+                if ( $self->{opt}{parentheses_w} == 1 ) {
                     unshift @choices, $unclosed ? ')' : '(';
                 }
-                elsif ( $self->{opt}{w_parentheses} == 2 ) {
+                elsif ( $self->{opt}{parentheses_w} == 2 ) {
                     push @choices, $unclosed ? ')' : '(';
                 }
                 $self->__print_select_statement( $sql, $table );
@@ -1438,10 +1381,10 @@ sub __read_table {
                 my @pre = ( $self->{info}{ok} );
                 unshift @pre, undef if $self->{opt}{sssc_mode};
                 my @choices = ( @{$self->{info}{avail_aggregate}}, map( '@' . $_, @{$sql->{print}{aggr_cols}} ) );
-                if ( $self->{opt}{h_parentheses} == 1 ) {
+                if ( $self->{opt}{parentheses_h} == 1 ) {
                     unshift @choices, $unclosed ? ')' : '(';
                 }
-                elsif ( $self->{opt}{h_parentheses} == 2 ) {
+                elsif ( $self->{opt}{parentheses_h} == 2 ) {
                     push @choices, $unclosed ? ')' : '(';
                 }
                 $self->__print_select_statement( $sql, $table );
