@@ -5,7 +5,7 @@ use warnings;
 use strict;
 use 5.010001;
 
-our $VERSION = '0.035';
+our $VERSION = '0.036';
 
 use Encode       qw( encode decode );
 use File::Find   qw( find );
@@ -15,71 +15,102 @@ use DBI                qw();
 use Encode::Locale     qw();
 use Term::Choose       qw( choose );
 use Term::Choose::Util qw( util_readline );
+use Term::ReadKey      qw( ReadMode );
 
 sub CLEAR_SCREEN () { "\e[1;1H\e[0J" }
 
 
 sub new {
     my ( $class, $info, $opt ) = @_;
-    #return $class if ref $class eq 'App::DBBrowser::DB';
     bless { info => $info, opt => $opt }, $class;
 }
 
 
-sub __get_username {
+sub __get_connect_data {
+    my ( $self, $db, $key ) = @_;
+    my $db_driver = $self->{info}{db_driver};
+    my $db_key = $db_driver . '_' . $db;
+    my $prompt = ucfirst( $key ) . ' :';
+    my $env_key = 'DBI_' . uc( $key );
+    return '' if $db_driver eq 'SQLite';
+    if ( $self->{opt}{db_host_port} ) {
+        return $self->{info}{login}{$db_key}{$key} if defined $self->{info}{login}{$db_key}{$key};
+        if ( length $self->{opt}{$db_key}{$key} ) {
+            say $prompt . $self->{opt}{$db_key}{$key};
+            return $self->{opt}{$db_key}{$key};
+        }
+        # Readline
+        my $new = util_readline( $prompt, { default => $self->{opt}{$db_driver}{$key} } ); #
+        $self->{info}{login}{$db_key}{$key} = $new;
+        return $new;
+    }
+    else {
+        return $ENV{$env_key}                 if $self->{opt}{'env_dbi_' . $key} && exists $ENV{$env_key};
+        return $self->{opt}{$db_driver}{$key} if exists $self->{opt}{$db_driver}{$key} && length $self->{opt}{$db_driver}{$key};
+    }
+    return;
+}
+
+
+sub __get_user {
     my ( $self, $db ) = @_;
     my $db_driver = $self->{info}{db_driver};
+    my $db_key = $db_driver . '_' . $db;
     return '' if $db_driver eq 'SQLite';
-    return $ENV{DBI_USER} if $self->{opt}{env_dbi_user};
-    my ( $user );
-    if ( $self->{opt}{db_login_once} ) {
-        $user = $self->{info}{login}{$db_driver}{user};
+    if ( $self->{opt}{db_user_pass} ) {
+        return $self->{info}{login}{$db_key}{user} if defined $self->{info}{login}{$db_key}{user};
+        if ( length $self->{opt}{$db_key}{user} ) {
+            say 'User :' . $self->{opt}{$db_key}{user};
+            return $self->{opt}{$db_key}{user};
+        }
+        # Readline
+        my $new;
+        if ( $^O eq 'MSWin32' ) {
+            print 'User: ';
+            ReadMode 1;
+            $new = <STDIN>;
+            ReadMode 0;
+            chomp $new;
+        }
+        else {
+            $new = util_readline( 'User: ', { default => $self->{opt}{$db_driver}{user} } ); #
+        }
+        $self->{info}{login}{$db_key}{user} = $new;
+        return $new;
     }
     else {
-        $user = $self->{info}{login}{$db_driver}{$db}{user};
+        return $self->{info}{login}{$db_driver}{user} if defined $self->{info}{login}{$db_driver}{user};
+        return $ENV{DBI_USER}                         if $self->{opt}{env_dbi_user} && exists $ENV{DBI_USER};
+        #return $self->{opt}{$db_key}{user}            if length $self->{opt}{$db_key}{user};
+        return $self->{opt}{$db_driver}{user}         if exists $self->{opt}{$db_driver}{user} && length $self->{opt}{$db_driver}{user};
+        # Readline
+        my $new = util_readline( 'User: ' );
+        $self->{info}{login}{$db_driver}{user} = $new;
+        return $new;
     }
-    # Readline
-    if ( ! defined $user ) {
-        print CLEAR_SCREEN;
-        print "DB: $db\n";
-        $user = util_readline( 'User: ' );
-    }
-
-    if ( $self->{opt}{db_login_once} ) {
-        $self->{info}{login}{$db_driver}{user} = $user;
-    }
-    else {
-        $self->{info}{login}{$db_driver}{$db}{user} = $user;
-    }
-    return $user;
 }
 
 
 sub __get_password {
     my ( $self, $db, $user ) = @_;
     my $db_driver = $self->{info}{db_driver};
+    my $db_key = $db_driver . '_' . $db;
     return '' if $db_driver eq 'SQLite';
-    return $ENV{DBI_PASS} if $self->{opt}{env_dbi_pass};
-    my ( $passwd );
-    if ( $self->{opt}{db_login_once} ) {
-        $passwd = $self->{info}{login}{$db_driver}{passwd};
+    if ( $self->{opt}{db_user_pass} ) {
+        return $self->{info}{login}{$db_key}{$user}{passwd} if defined $self->{info}{login}{$db_key}{$user}{passwd};
+        # Readline
+        my $passwd = util_readline( 'Password: ', { no_echo => 1 } );
+        $self->{info}{login}{$db_key}{$user}{passwd} = $passwd;
+        return $passwd;
     }
     else {
-        $passwd = $self->{info}{login}{$db_driver}{$db}{passwd};
+        return $self->{info}{login}{$db_driver}{$user}{passwd} if defined $self->{info}{login}{$db_driver}{$user}{passwd};
+        return $ENV{DBI_PASS}                                  if $self->{opt}{env_dbi_pass} && exists $ENV{DBI_PASS};
+        # Readline
+        my $passwd = util_readline( 'Password: ', { no_echo => 1 }  );
+        $self->{info}{login}{$db_driver}{$user}{passwd} = $passwd;
+        return $passwd;
     }
-    # Readline
-    if ( ! defined $passwd ) {
-        print CLEAR_SCREEN;
-        print "DB: $db\nUser: $user\n";
-        $passwd = util_readline( 'Password: ', { no_echo => 1 } );
-    }
-    if ( $self->{opt}{db_login_once} ) {
-        $self->{info}{login}{$db_driver}{passwd} = $passwd;
-    }
-    else {
-        $self->{info}{login}{$db_driver}{$db}{passwd} = $passwd;
-    }
-    return $passwd;
 }
 
 
@@ -89,16 +120,18 @@ sub get_db_handle {
     return if ! defined $db && $db_driver eq 'SQLite';
     my $db_key = $db_driver . '_' . $db;
     my $db_arg = {};
-    for my $option ( @{$self->{info}{$db_driver}{options}} ) {
-        next if $option =~ /^_/;
+    for my $option ( sort keys %{$self->{opt}{$db_driver}} ) {
+        next if $option !~ /^\Q$self->{info}{connect_opt_pre}{$db_driver}\E/;
         $db_arg->{$option} = $self->{opt}{$db_key}{$option} // $self->{opt}{$db_driver}{$option};
     }
+    print CLEAR_SCREEN;
+    print "DB: $db\n";
+    my $host = $self->__get_connect_data( $db, 'host' );
+    my $port = $self->__get_connect_data( $db, 'port' );
     my $dsn = 'dbi:' . $db_driver . ':dbname=' . $db;
-    $dsn .= ';host=' . $db_arg->{host} if $db_arg->{host};
-    $dsn .= ';port=' . $db_arg->{port} if $db_arg->{port};
-    delete $db_arg->{host};
-    delete $db_arg->{port};
-    my $user   = $self->__get_username( $db );
+    $dsn .= ';host=' . $host if length $host;
+    $dsn .= ';port=' . $port if length $port;
+    my $user   = $self->__get_user( $db );
     my $passwd = $self->__get_password( $db, $user );
     my $dbh = DBI->connect( $dsn, $user, $passwd, {
         PrintError => 0,
