@@ -5,7 +5,7 @@ use warnings;
 use strict;
 use 5.010001;
 
-our $VERSION = '0.035';
+our $VERSION = '0.036';
 
 use Encode                qw( encode );
 use File::Basename        qw( basename );
@@ -25,12 +25,71 @@ sub new {
 }
 
 
-sub options {
-    my ( $self ) = @_;
+sub defaults {
+    my ( $self, @keys ) = @_;
+    my $defaults = {
+        db_drivers           => [ 'SQLite', 'mysql', 'Pg' ],
+        db_host_port         => 1,
+        db_user_pass         => 1,
+        env_dbi_user         => 0,
+        env_dbi_pass         => 0,
+        env_dbi_host         => 0,
+        env_dbi_port         => 0,
+        menus_memory         => 0,
+        table_expand         => 1,
+        keep_header          => 0,
+        lock_stmt            => 0,
+        max_rows             => 50_000,
+        metadata             => 0,
+        mouse                => 0,
+        min_col_width        => 30,
+        operators            => [ "REGEXP", " = ", " != ", " < ", " > ", "IS NULL", "IS NOT NULL" ],
+        parentheses_w        => 0,
+        parentheses_h        => 0,
+        progress_bar         => 20_000,
+        regexp_case          => 0,
+        sssc_mode            => 0,
+        tab_width            => 2,
+        undef                => '',
+        binary_string        => 'BNRY',
+        thsd_sep             => ',',
+        #add_header           => 0,
+        #choose_columns       => 0,
+        SQLite => {
+            _reset_cache_cmdline_only  => 0,
+            sqlite_unicode             => 1,
+            sqlite_see_if_its_a_number => 1,
+            binary_filter              => 0,
+            dirs_sqlite_search         => undef,
+        },
+        mysql => {
+            user              => undef,
+            host              => undef,
+            port              => undef,
+            mysql_enable_utf8 => 1,
+            binary_filter     => 0,
+        },
+        Pg => {
+            user           => undef,
+            host           => undef,
+            port           => undef,
+            pg_enable_utf8 => -1,
+            binary_filter  => 0,
+        },
+    };
+    die "To many keys: @keys"              if @keys >  2;
+    return $defaults->{$keys[0]}           if @keys == 1;
+    return $defaults->{$keys[0]}{$keys[1]} if @keys == 2;
+    return $defaults;
+}
+
+
+sub set_options {
+    my ( $self, $opt ) = @_;
     my $menus = [
-        [ 'db_defaults',    "- DB Defaults" ],
+        [ '_db_defaults',   "- DB Defaults" ],
         [ 'db_drivers',     "- DB Drivers" ],
-        [ 'db_login_once',  "- DB Login" ],
+        [ '_db_connect',    "- DB Login" ],
         [ '_env_dbi',       "- ENV DBI" ],
         [ '_enchant',       "- Enchant" ],
         [ 'lock_stmt',      "- Lock" ],
@@ -48,20 +107,24 @@ sub options {
     ];
     my $sub_menus = {
        _enchant      => [
-            [ 'keep_db_choice',     "- Choose Database", [ 'Simple', 'Memory' ] ],
-            [ 'keep_schema_choice', "- Choose Schema",   [ 'Simple', 'Memory' ] ],
-            [ 'keep_table_choice',  "- Choose Table",    [ 'Simple', 'Memory' ] ],
-            [ 'table_expand',       "- Print  Table",    [ 'Simple', 'Expand' ] ],
-            [ 'keep_header',        "- Table Header",    [ 'Simple', 'Each page' ] ],
+            [ 'menus_memory',  "- Menus",        [ 'Simple', 'Memory' ] ],
+            [ 'table_expand',  "- Print  Table", [ 'Simple', 'Expand' ] ],
+            [ 'keep_header',   "- Table Header", [ 'Simple', 'Each page' ] ],
         ],
         _parentheses => [
-            [ 'w_parentheses', "- Parentheses in WHERE",     [ 'NO', '(YES', 'YES(' ] ],
-            [ 'h_parentheses', "- Parentheses in HAVING TO", [ 'NO', '(YES', 'YES(' ] ],
+            [ 'parentheses_w', "- Parentheses in WHERE",     [ 'NO', '(YES', 'YES(' ] ],
+            [ 'parentheses_h', "- Parentheses in HAVING TO", [ 'NO', '(YES', 'YES(' ] ],
         ],
         _env_dbi     => [
             [ 'env_dbi_user', "- Use DBI_USER", [ 'NO', 'YES' ] ],
             [ 'env_dbi_pass', "- Use DBI_PASS", [ 'NO', 'YES' ] ],
+            [ 'env_dbi_host', "- Use DBI_HOST", [ 'NO', 'YES' ] ],
+            [ 'env_dbi_port', "- Use DBI_PORT", [ 'NO', 'YES' ] ],
         ],
+        _db_connect  => [
+            [ 'db_host_port', "- Ask host/port per DB", [ 'NO', 'YES' ] ],
+            [ 'db_user_pass', "- Ask user/pass per DB", [ 'NO', 'YES' ] ],
+        ]
     };
     my $path = '  Path';
     my @pre = ( undef, $self->{info}{_continue}, $self->{info}{_help}, $path );
@@ -86,14 +149,14 @@ sub options {
         }
         if ( ! defined $key ) {
             if ( $self->{info}{write_config} ) {
-                $self->__write_config_file( $self->{info}{config_file} );
+                $self->__write_config_files();
                 delete $self->{info}{write_config};
             }
             exit();
         }
         elsif ( $key eq $self->{info}{_continue} ) {
             if ( $self->{info}{write_config} ) {
-                $self->__write_config_file( $self->{info}{config_file} );
+                $self->__write_config_files();
                 delete $self->{info}{write_config};
             }
             return $self->{opt};
@@ -101,7 +164,6 @@ sub options {
         elsif ( $key eq $self->{info}{_help} ) {
             require Pod::Usage;
             Pod::Usage::pod2usage( {
-                -input => 'bin/db-browser',
                 -exitval => 'NOEXIT',
                 -verbose => 2 } );
         }
@@ -147,12 +209,12 @@ sub options {
             $self->__opt_choose_index( $key, $prompt, $list );
         }
         elsif ( $key eq 'metadata' ) {
-            my $list = $self->{info}{yes_no};
+            my $list = [ 'NO', 'YES' ];
             my $prompt = 'Enable Metadata';
             $self->__opt_choose_index( $key, $prompt, $list );
         }
         elsif ( $key eq 'regexp_case' ) {
-            my $list = $self->{info}{yes_no};
+            my $list = [ 'YES', 'NO' ];
             my $prompt = 'REGEXP case sensitiv';
             $self->__opt_choose_index( $key, $prompt, $list );
         }
@@ -160,16 +222,15 @@ sub options {
             my $sub_menu = $sub_menus->{$key};
             $self->__opt_choose_multi( $sub_menu );
         }
-        elsif ( $key eq 'db_login_once' ) {
-            my $list = [ 'per-DB', 'once' ];
-            my $prompt = 'Ask for credentials';
-            $self->__opt_choose_index( $key, $prompt, $list );
+        elsif ( $key eq '_db_connect' ) {
+            my $sub_menu = $sub_menus->{$key};
+            $self->__opt_choose_multi( $sub_menu );
         }
         elsif ( $key eq '_env_dbi' ) {
             my $sub_menu = $sub_menus->{$key};
             $self->__opt_choose_multi( $sub_menu );
         }
-        elsif ( $key eq 'db_defaults' ) {
+        elsif ( $key eq '_db_defaults' ) {
             $self->database_setting();
         }
         elsif ( $key eq 'sssc_mode' ) {
@@ -265,9 +326,8 @@ sub __opt_number_range {
 sub __opt_readline {
     my ( $self, $key, $prompt ) = @_;
     my $current = $self->{opt}{$key};
-    $prompt .= ' ["' . $current . '"]: ';
     # Readline
-    my $choice = util_readline( $prompt );
+    my $choice = util_readline( $prompt . ': ', { default => $current } );
     return if ! defined $choice;
     $self->{opt}{$key} = $choice;
     $self->{info}{write_config}++;
@@ -296,6 +356,8 @@ sub database_setting {
         $db_driver = $self->{info}{db_driver};
         $section   = $db_driver . '_' . $db;
         for my $key ( keys %{$self->{opt}{$db_driver}} ) {
+            next if $key =~ /^(?:host|port|user)\z/; #
+            next if $key eq 'dirs_sqlite_search';
             $self->{opt}{$section}{$key} //= $self->{opt}{$db_driver}{$key};
         }
     }
@@ -305,33 +367,32 @@ sub database_setting {
         SQLite => [
             [ 'sqlite_unicode',             "- Unicode" ],
             [ 'sqlite_see_if_its_a_number', "- See if its a number" ],
-            [ '_binary_filter',             "- Binary Filter" ],
         ],
         mysql => [
-            [ 'host',              "- Host" ],
-            [ 'port',              "- Port" ],
             [ 'mysql_enable_utf8', "- Enable utf8" ],
-            [ '_binary_filter',    "- Binary Filter" ],
         ],
         Pg => [
-            [ 'host',           "- Host" ],
-            [ 'port',           "- Port" ],
             [ 'pg_enable_utf8', "- Enable utf8" ],
-            [ '_binary_filter', "- Binary Filter" ],
         ],
     };
+    if ( $db_driver =~ /^(?:mysql|Pg)\z/ ) {
+        unshift @{$menus->{$db_driver}}, [ 'host', "- Host" ], [ 'port', "- Port" ] if $self->{opt}{db_host_port};
+        unshift @{$menus->{$db_driver}}, [ 'user', "- User" ];
+    }
+    if ( ! $db && $db_driver eq 'SQLite' ) {
+        push @{$menus->{$db_driver}}, [ 'dirs_sqlite_search', "- Default DB dirs" ];
+    }
+    push @{$menus->{$db_driver}}, [ 'binary_filter', "- Binary Filter" ], [ '_reset', "  RESET" ];
     my $prompt;
     if ( defined $db ) {
         $prompt = 'DB: "' . ( $db_driver eq 'SQLite' ? basename( $db ) : $db ) . '"';
     }
     else {
         $prompt = 'Driver: ' . $db_driver;
-        push @{$menus->{SQLite}}, [ '_sqlite_search_dir', "- Default DB dirs" ];
     }
     my @pre = ( undef, $self->{info}{_confirm} );
     my @real = map { $_->[1] } @{$menus->{$db_driver}};
     my $choices = [ @pre, @real ];
-    push @$choices, "  RESET" if defined $db;
 
     DB_OPTION: while ( 1 ) {
         # Choose
@@ -344,58 +405,75 @@ sub database_setting {
         if ( $idx <= $#pre ) {
             $key = $pre[$idx];
         }
-        elsif ( $idx == @pre + @real ) {
-            for my $key ( keys %{$self->{opt}{$section}} ) {
-                $self->{opt}{$section}{$key} = undef;
-            }
-            $self->{info}{write_config}++;
-            next;
-        }
         else {
             $idx -= @pre;
             $key = $menus->{$db_driver}[$idx][0];
-            die $key if ! exists $self->{opt}{$db_driver}{$key};
         }
         if ( ! defined $key ) {
-            if ( $self->{info}{write_config} ) {
-                $self->{opt} = clone( $orig );
-            }
+            $self->{opt} = clone( $orig ) if $self->{info}{write_config};
             return;
         }
+        if ( $key eq '_reset' ) {
+            if ( $db ) {
+                delete $self->{opt}{$section};
+            }
+            else {
+                my @dbs = ();
+                for my $section ( keys %{$self->{opt}} ) {
+                    push @dbs, $1 if $section =~ /^\Q$db_driver\E_(.+)\z/;
+                }
+                my $dlt = choose_a_subset( [ '*' . $db_driver, sort @dbs ], { p_new => 'Reset: ' } );
+                next DB_OPTION if ! defined $dlt;
+                next DB_OPTION if ! defined $dlt->[0];
+                for my $db ( @$dlt ) {
+                    if ( $db eq '*' . $db_driver ) {
+                        $self->{opt}{$db_driver} = $self->defaults( $db_driver );
+                    }
+                    else {
+                        my $section = $db_driver . '_' . $db;
+                        delete $self->{opt}{$section};
+                    }
+                }
+            }
+            $self->{info}{write_config}++;
+            next DB_OPTION;
+        }
         if ( $key eq $self->{info}{_confirm} ) {
             if ( $self->{info}{write_config} ) {
-                $self->__write_config_file( $self->{info}{config_file} );
+                $self->__write_config_files();
                 delete $self->{info}{write_config};
                 return 1;
             }
             return;
         }
+        my $yes_no = [ 'NO', 'YES' ];
+
         if ( $db_driver eq "SQLite" ) {
             if ( $key eq 'sqlite_unicode' ) {
-                my $list = $self->{info}{yes_no};
                 my $prompt = 'Unicode';
-                $self->__db_opt_choose_index( $section, $key, $prompt, $list );
+                $self->__db_opt_choose_index( $section, $key, $prompt, $yes_no );
             }
             elsif ( $key eq 'sqlite_see_if_its_a_number' ) {
-                my $list = $self->{info}{yes_no};
                 my $prompt = 'See if its a number';
-                $self->__db_opt_choose_index( $section, $key, $prompt, $list );
+                $self->__db_opt_choose_index( $section, $key, $prompt, $yes_no );
             }
-            elsif ( $key eq '_binary_filter' ) {
-                my $list = $self->{info}{yes_no};
+            elsif ( $key eq 'binary_filter' ) {
                 my $prompt = 'Enable Binary Filter';
-                $self->__db_opt_choose_index( $section, $key, $prompt, $list );
+                $self->__db_opt_choose_index( $section, $key, $prompt, $yes_no );
             }
-            elsif ( $key eq '_sqlite_search_dir' ) {
+            elsif ( $key eq 'dirs_sqlite_search' ) {
                 $self->__db_opt_choose_dirs( $section, $key, $prompt );
             }
             else { die "Unknown key: $key" }
         }
         elsif ( $db_driver eq "mysql" ) {
             if ( $key eq 'mysql_enable_utf8' ) {
-                my $list = $self->{info}{yes_no};
                 my $prompt = 'Enable utf8';
-                $self->__db_opt_choose_index( $section, $key, $prompt, $list );
+                $self->__db_opt_choose_index( $section, $key, $prompt, $yes_no );
+            }
+            elsif ( $key eq 'user' ) {
+                my $prompt = 'User';
+                $self->__db_opt_readline( $section, $key, $prompt );
             }
             elsif ( $key eq 'host' ) {
                 my $prompt = 'Host';
@@ -405,20 +483,23 @@ sub database_setting {
                 my $prompt = 'Port';
                 $self->__db_opt_readline( $section, $key, $prompt );
             }
-            elsif ( $key eq '_binary_filter' ) {
-                my $list = $self->{info}{yes_no};
+            elsif ( $key eq 'binary_filter' ) {
                 my $prompt = 'Enable Binary Filter';
-                $self->__db_opt_choose_index( $section, $key, $prompt, $list );
+                $self->__db_opt_choose_index( $section, $key, $prompt, $yes_no );
             }
             else { die "Unknown key: $key" }
         }
         elsif ( $db_driver eq "Pg" ) {
             if ( $key eq 'pg_enable_utf8' ) {
                 my $prompt = 'Enable utf8';
-                my $list = [ @{$self->{info}{yes_no}}, 'AUTO' ];
+                my $list = [ @{$yes_no}, 'AUTO' ];
                 $self->__db_opt_choose_index( $section, $key, $prompt, $list );
                 $self->{opt}{$section}{$key} = -1 if $self->{opt}{$section}{$key} == 2;
             }
+            elsif ( $key eq 'user' ) {
+                my $prompt = 'User';
+                $self->__db_opt_readline( $section, $key, $prompt );
+            }
             elsif ( $key eq 'host' ) {
                 my $prompt = 'Host';
                 $self->__db_opt_readline( $section, $key, $prompt );
@@ -427,10 +508,9 @@ sub database_setting {
                 my $prompt = 'Port';
                 $self->__db_opt_readline( $section, $key, $prompt );
             }
-            elsif ( $key eq '_binary_filter' ) {
-                my $list = $self->{info}{yes_no};
+            elsif ( $key eq 'binary_filter' ) {
                 my $prompt = 'Enable Binary Filter';
-                $self->__db_opt_choose_index( $section, $key, $prompt, $list );
+                $self->__db_opt_choose_index( $section, $key, $prompt, $yes_no );
             }
             else { die "Unknown key: $key" }
         }
@@ -471,9 +551,8 @@ sub __db_opt_choose_dirs {
 sub __db_opt_readline {
     my ( $self, $section, $key, $prompt ) = @_;
     my $current = $self->{opt}{$section}{$key};
-    $prompt .= ' ["' . $current . '"]: ';
     # Readline
-    my $choice = util_readline( $prompt );
+    my $choice = util_readline( $prompt . ': ', { default => $current } );
     return if ! defined $choice;
     $self->{opt}{$section}{$key} = $choice;
     $self->{info}{write_config}++;
@@ -481,45 +560,66 @@ sub __db_opt_readline {
 }
 
 
-sub __write_config_file {
-    my ( $self, $file ) = @_;
+sub __write_config_files {
+    my ( $self ) = @_;
+    my $regexp_drivers = join '|', map quotemeta, @{$self->defaults( qw( db_drivers ) )};
+    my $fmt = $self->{info}{conf_file_fmt};
     my $tmp = {};
     for my $section ( sort keys %{$self->{opt}} ) {
-        if ( ref( $self->{opt}{$section} ) eq 'HASH' ) {
+        if ( $section =~ /^($regexp_drivers)(?:_(.+))?\z/ ) {
+            die $section if ref( $self->{opt}{$section} ) ne 'HASH';
+            my ( $db_driver, $conf_sect ) = ( $1, $2 );
+            $conf_sect //= '*' . $db_driver;
             for my $key ( keys %{$self->{opt}{$section}} ) {
-                $tmp->{$section}{$key} = $self->{opt}{$section}{$key};
+                next if $key =~ /^_/;
+                $tmp->{$db_driver}{$conf_sect}{$key} = $self->{opt}{$section}{$key};
             }
         }
         else {
+            die $section if ref( $self->{opt}{$section} ) eq 'HASH';
+            my $generic = $self->{info}{sect_generic};
             my $key = $section;
-            my $section = $self->{info}{sect_generic};
-            $tmp->{$section}{$key} = $self->{opt}{$key};
+            next if $key =~ /^_/;
+            $tmp->{$generic}{$key} = $self->{opt}{$key};
         }
     }
-    $self->write_json( $file, $tmp );
+    for my $name ( keys %$tmp ) {
+        $self->write_json( sprintf( $fmt, $name ), $tmp->{$name}  );
+    }
+
 }
 
 
-sub read_config_file {
-    my ( $self, $file ) = @_;
-    my $tmp = $self->read_json( $file );
-    for my $section ( keys %$tmp ) {
-        for my $key ( keys %{$tmp->{$section}} ) {
-            if ( $section eq $self->{info}{sect_generic} ) {
-                $self->{opt}{$key} = $tmp->{$section}{$key};
-            }
-            else {
-                $self->{opt}{$section}{$key} = $tmp->{$section}{$key};
+sub read_config_files {
+    my ( $self ) = @_;
+    $self->{opt} = $self->defaults();
+    my $fmt = $self->{info}{conf_file_fmt};
+    for my $db_driver ( @{$self->defaults( qw( db_drivers ) )} ) {
+        my $file = sprintf( $fmt, $db_driver );
+        if ( -f $file && -s $file ) {
+            my $tmp = $self->read_json( $file );
+            for my $conf_sect ( keys %$tmp ) {
+                my $section = $db_driver . ( $conf_sect =~ /^\*(?:$db_driver)\z/ ? '' : '_' . $conf_sect );
+                for my $key ( keys %{$tmp->{$conf_sect}} ) {
+                    $self->{opt}{$section}{$key} = $tmp->{$conf_sect}{$key} if exists $self->{opt}{$db_driver}{$key};
+                }
             }
         }
     }
+    my $file =  sprintf( $fmt, $self->{info}{sect_generic} );
+    if ( -f $file && -s $file ) {
+        my $tmp = $self->read_json( $file );
+        for my $key ( keys %$tmp ) {
+            $self->{opt}{$key} = $tmp->{$key} if exists $self->{opt}{$key};
+        }
+    }
     return $self->{opt};
 }
 
 
 sub write_json {
     my ( $self, $file, $h_ref ) = @_;
-    my $json = JSON::XS->new->pretty->canonical->encode( $h_ref );
+    my $json = JSON::XS->new->utf8( 1 )->pretty->canonical->encode( $h_ref );
     open my $fh, '>', encode( 'locale_fs', $file ) or die $!;
     print $fh $json;
     close $fh;
@@ -533,7 +633,12 @@ sub read_json {
     my $json = do { local $/; <$fh> };
     close $fh;
     my $h_ref = {};
-    $h_ref = decode_json( $json ) if $json;
+    if ( ! eval {
+        $h_ref = decode_json( $json ) if $json;
+        1 }
+    ) {
+        die "In '$file':\n$@";
+    }
     return $h_ref;
 }
 
